from dwll.memory import memory
from dwll.memory import constants as C

class TopMemory(memory.Memory):

    """

    Top Memory
    ===================

    Description
        Helper/Manager que funciona como especializacion de Memory
        para proceder con limpiezas de memoria escalares.

    """

    def clean_by_step(self, request, step=C.MAIN):
        """

        Clean By Step

        Description
            Limpia la memoria por pasos, la limpieza des pasos iniciales
            hace que los posteriores pasos tambien sean limpiados.

        :param request:
        :param step:
        :return:
        """

        if step == C.MAIN:
            self.clean_main(request)

    def clean_main(self, request):
        """

        Clean Step 0

        Description
            Limpieza paso 0 (HOME)

        :param request:
        :return:
        """
        self.clean(request, C.MAIN)

top = TopMemory()



from . import messages
from . import configuration

class UploadPicturesUtil:
    """

    Upload Pictures Utils

    """

    @staticmethod
    def profile_picture(instance, filename):
        name, file_extention = os.path.splitext(filename)
        name = 'user-{}.{}'.format(instance.code, file_extention)
        return "pictures/users/{}/{}".format(instance.code, name)

    @staticmethod
    def validate_small(image_field):
        """

        Validate Upload Image Size

        """
        ext = os.path.splitext(image_field.name)[1]  # [0] returns path+filename

        valid_extensions = ['.jpg', '.png', '.jpeg']

        if not ext.lower() in valid_extensions:
            raise ValidationError(
                messages.get_message('pictures.small.ext.error'))

        file_size = image_field.file.size

        limit_kb = configuration.get_integer(
            'pictures.small.maxlength.kb', default=200)

        if file_size > limit_kb * KILOBYTE:
            raise ValidationError('{} {}KB'.format(
                messages.get_message('pictures.small.maxlength.error'), limit_kb))



# -*- coding: utf-8 -*-
"""
.. module:: dbu-impexp
   :platform: Unix, Windows
   :synopsis: Import Export resources for dbu module

.. moduleauthor:: Diego Gonzalez <dgonzalez.jim@gmail.com>

"""
from import_export import resources

from . import models

class ConfigurationResource(resources.ModelResource):
    """

    Configuration Resource
    ===================

    Description
        Recurso para exportar o importar datos del modelo
        Configuration.

    """

    class Meta:
        model = models.Configuration

class MessageResource(resources.ModelResource):
    """

    Message Resource
    ===================

    Description
        Recurso para exportar o importar datos del modelo
        Message.

    """

    class Meta:
        model = models.Message

class MessageTraductionResource(resources.ModelResource):
    """

    Message Resource
    ===================

    Description
        Recurso para exportar o importar datos del modelo
        MessageTraduction.

    """

    class Meta:
        model = models.MessageTraduction


# -*- coding: utf-8 -*-
"""
.. module:: dbu-messages
   :platform: Unix, Windows
   :synopsis: Manejador de mensajes de uso comun y principal

.. moduleauthor:: Diego Gonzalez <dgonzalez.jim@gmail.com>

"""
from . import models
from .languages import languages

class MessageManager:
    """

    Messages Manager
    ===================

    Description
        Manejador de mensajes, mantiene en sesion los mensajes
        que ya fueron consultados, ademas permite internacionalizacion
        de los mismos.

    """

    DEFAULT_MSG = '-NO MESSAGE AVAILABLE-'

    def __init__(self, language=None):
        self.language = language

        if self.language:
            if self.language == languages.get_default():
                self.is_default = True
            else:
                self.is_default = False
        else:
            self.is_default = True

    def get_message(self, name, default=None):
        """

        Get Message

        Description
            Permite consultar un mensaje por su nombre, devuelve incluida
            su internacionalizacion, si no existe devuelve un mensaje por
            defecto y crea dicho mensaje. Almacena en memoria de servidor.

        :return
            String -- El texto del mensaje

        """
        message = name
        
        if self.is_default:
            language_name = 'default'
        else:
            language_name = self.language if isinstance(self.language, str) else self.language.name

        if hasattr(get_message, 'messages'):
            messages_dict = get_message.messages
        else:
            messages_dict = {}
        
        if '%s.%s' % (language_name, name) in messages_dict:
            message = messages_dict['%s.%s' % (language_name, name)]
        else:
            try:
                if self.is_default or language_name == languages.DEFAULT:
                    messages = models.Message.objects.get_active(name = name, published=True)
                else:
                    messages = models.MessageTraduction.objects.get_active(message__name = name,
                                                                           language__name = self.language)

                if messages.count() > 0:
                    message = messages.first().text
                    messages_dict['%s.%s' % (language_name, name)] = message
                    get_message.messages = messages_dict
                else:
                    if self.create_default(name, self.language, default):
                        message = default if default else name
            except Exception as e:
                print('[messages] Error al obtener el mensaje: %s' % str(e))
                message = None

        if message == self.DEFAULT_MSG:
            return name
        
        return message

    def create_default(self, name, language, default=None):
        """

        Create Default

        Description
            Permite crear un mensaje por defecto en caso de que no exista,
            asi estara disponible para que el administrador simplemente
            llene el texto correspondiente y sus traducciones.

        """
        try:
            if name is None or len(name) > 100:
                return False
            else:
                if self.is_default:
                    msg = models.Message.objects.create(name=name, text=default if default else name) #self.DEFAULT_MSG
                else:
                    try:
                        message = models.Message.objects.get(name = name)
                    except models.Message.DoesNotExist:
                        message = models.Message.objects.create(name=name, text=default if default else name)
                    
                    models.MessageTraduction.objects.create(message=message, text=default if default else name,
                                                            language=languages.get_by_name(language))

            return True
        except Exception as e:
            import traceback
            traceback.print_exc()
            return False

def get_message(name, language=None, default=None):
    """

    Get Message

    Description
        Acceso rapido a los mensajes

    :param name: El nombre unico del mensaje
    :param language: El lenguaje del mensaje
    :return: String -- El texto del mensaje
    """
    return MessageManager(language).get_message(name, default=default)

def get_full_message(request, name, default=None):
    """

    Get Message Complete

    Description
        Acceso rapido a los mensajes con internacionalizacion

    :param request:
    :param name: El nombre unico del mensaje
    :return: String -- El texto del mensaje
    """
    return MessageManager(languages.get_language(request)).get_message(name, default=default)

def format_message(message, *args):
    """
    
    Format message

    Description
        Agrega variables a un mensaje
    
    :param message:
    :param *args: Lista de valores a reempazarse en el mensaje
    :return message: Mensaje evaluado
    """
    if not args:
        return message

    for index, option in enumerate(args):
        message = message.replace('{%d}' % index, option)

    return message



from django.template.loader import render_to_string
from pathlib import Path

import shutil
import os
import traceback

from .memory import dinamic

BASE_DIR = Path(__file__).resolve().parent

class Template:

    PYTHON_FILE = 'py'
    HTML_FILE = 'html'

    def __init__(self, app_name, model_name, template_name, ext='py', do_replace=False, 
                 final_template_name=None):
        self.app_name = app_name
        self.model_name = model_name
        self.template_name = template_name
        self.ext = ext
        self.current_path = os.path.abspath(os.getcwd())
        self.do_replace = do_replace
        self.final_template_name = final_template_name

    def create_dir(self, dirname, secondary='', third=''):
        try:
            path = os.path.join(self.current_path, dirname, secondary, third)
            if not os.path.isdir(path):
                os.mkdir(path)
        except:
            pass

    def render(self):
        """
        Renderiza el template dado en el archivo solicitado
        """
        try:
            template_file = '%s_template.html' % self.template_name
            destiny_name = self.final_template_name\
                if self.final_template_name else self.template_name

            if self.ext == self.PYTHON_FILE:
                self.create_dir(self.app_name)
                rendered = render_to_string(template_file, {'app_name': self.app_name, 
                                                            'model_name': self.model_name})
                to_path = os.path.join(self.current_path, self.app_name,
                    '%s.py' % ("__init__" if  destiny_name == 'init' else destiny_name))
                
                print('- Creando archivo Python:', destiny_name)
                
                with open(to_path, 'w') as f:
                    f.write(rendered)
            else:
                
                from_path = os.path.join(BASE_DIR, 'templates', template_file)
                if destiny_name in ['login','logout']:
                    self.create_dir(self.app_name, 'templates', 'account')
                    to_path = os.path.join(self.current_path, self.app_name, 'templates', 
                        'account', '%s.html' % destiny_name)
                elif destiny_name in ['list','form']:
                    self.create_dir(self.app_name, 'templates', self.app_name)
                    to_path = os.path.join(self.current_path, self.app_name, 'templates', 
                        self.app_name, '%s_%s.html' % (self.model_name, destiny_name))
                else:
                    self.create_dir(self.app_name, 'templates')
                    to_path = os.path.join(self.current_path, self.app_name, 'templates', 
                        '%s.html' % destiny_name)
                    
                print('- Creando archivo HTML:', destiny_name)
                
                shutil.copyfile(from_path, to_path)
                
                if self.do_replace:
                    self.replace_html_var(to_path)
        except Exception as e:
            print('Generation Error:',e)
            traceback.print_exc()
            
    def replace_html_var(self, final_path):
        dinamic.replace_html(final_path, '[ITEM]', self.model_name)

class Generator:

    def __init__(self, app_name, model_name=None):
        self.app_name = app_name
        self.model_name = model_name

class AppGenerator(Generator):

    def generate(self):
        Template(self.app_name, self.model_name, 'init').render()
        Template(self.app_name, self.model_name, 'apps').render()
        Template(self.app_name, self.model_name, 'admin').render()
        Template(self.app_name, self.model_name, 'models').render()
        Template(self.app_name, self.model_name, 'signals').render()
        Template(self.app_name, self.model_name, 'tests').render()
        Template(self.app_name, self.model_name, 'urls').render()
        Template(self.app_name, self.model_name, 'views').render()
        Template(self.app_name, self.model_name, 'forms').render()
        Template(self.app_name, self.model_name, 'locustfile').render()

class AppTemplatesGenerator(Generator):

    def generate(self):
        Template(self.app_name, None, 'base', 'html').render()
        Template(self.app_name, None, 'header', 'html').render()
        Template(self.app_name, None, 'footer', 'html').render()
        Template(self.app_name, None, 'login', 'html').render()
        Template(self.app_name, None, 'logout', 'html').render()
        Template(self.app_name, None, 'home', 'html').render() 
        Template(self.app_name, None, 'messages', 'html').render()
        
        if self.model_name:
            Template(self.app_name, self.model_name, 'list', 'html', do_replace=True).render()
            Template(self.app_name, self.model_name, 'form', 'html', do_replace=True).render()
            Template(self.app_name, self.model_name, 'menulist', 'html', 
                     do_replace=True, final_template_name='menu').render()
        else:
            Template(self.app_name, None, 'menu', 'html').render()    

def run_generator_engine(option, app_name=None, model_name=None):

    if option == 'app':
        if app_name:
            AppGenerator(app_name, model_name).generate()
            AppTemplatesGenerator(app_name, model_name).generate()


# -*- coding: utf-8 -*-
"""
.. module:: commons
   :platform: Unix, Windows
   :synopsis: Ajax base class

.. moduleauthor:: Diego Gonzalez <dgonzalez.jim@gmail.com>

"""

import uuid

class AjaxBase:

    params = {}
    exist_all = True

    DEBUG = False

    def ID(self):
        """

        Description
            Devuelve un ID Unico a ser usado por el resto de metodos
            para evitar obtener parametros con valores cambiados

        :return:
        """
        id = uuid.uuid4().int & (1<<32)-1
        id = str(id)[:8]
        if self.DEBUG:print('==> {AJAX REQUEST #', id,'}')
        return id

    def GET(self, id, request, attributes):
        """

        GET Params

        Description
            Dada una lista de atributos, construye un diccionario
            de los parametros pasados por GET

        @param attributes List - String
        """
        for p in attributes:
            if p in request.GET:
                self.params['%s-%s' % (p, id)] = request.GET.get(p)
            else:
                if self.DEBUG:print('<== [AjaxBase.GET] No fue dado', p)
                self.exist_all = False

    def E(self, id, names=None):
        """

        Exist

        Description
            Existe una lista de parametros en params

        :param names:
        :return: True Si existen todos, False si no existe al menos uno
        """
        if names:
            for name in names:
                key = '%s-%s' % (name, id)
                if key not in self.params:
                    return False
            return True
        else:
            return self.exist_all

    def V(self, id, name):
        """

        Value

        Description
            Devuelve el valor de un parametro

        :param name:
        :return:
        """
        key = '%s-%s' % (name, id)
        if key in self.params:
            return self.params[key]
        return None

    def fail(self, error_msg='', method='', info=None):
        """

        Fail

        Description
            Respuesta de error

        :param error_msg:
        :param method:
        :return:
        """
        info = info if info else self.params
        return {
            'success': False,
            'message': error_msg,
            'info':info,
            'method': method,
        }

    def success(self, info=None, method=''):
        """

        Fail

        Description
            Respuesta de error

        :param info: Dictionary
        :param method:
        :return:
        """
        info = info if info else self.params
        return {
            'success': True,
            'message':'OK',
            'info':info,
            'method':method
        }




from django.http import HttpResponseRedirect
from django.utils.safestring import mark_safe
from django.contrib import messages

from dwll import messages as __
from . import constants as C

class AdminMixinBase:

    """

    Model Admin Mixin
    ===================

    Description
        Esta clase permite manejar un manager comun para todos los administradores
        de modelos del sistema, tiene las funciones basicas de un ModelAdmin sobrecargadas.

    """

    def get_form(self, request, obj=None, **kwargs):
        """

        Get Form

        Description
            Sobrecarga del metodo get_form de ModelAdmin para excluir de los usuarios
            NO super usuarios, los campos de auditoria

        :param request:
        :param obj:
        :param kwargs:
        :return:
        """
        if not request.user.is_superuser:
            self.exclude = ['status', 'creation_date', 'creation_user', 'modification_date', 'modification_user']
        return super(AdminMixinBase, self).get_form(request, obj, **kwargs)

    def save_model(self, request, obj, form, change):
        """

        Save Model

        Description
            Sobrecarga del metodo save_model de ModelAdmin para almacenar el usaurio que
            esta realizando una afectacion a un modelo desde el Admin Django.

        :param request:
        :param obj:
        :param form:
        :param change:
        :return:
        """
        obj.save(request = request)

    def get_dual_methods(self, obj=None, disable_color=C.DISABLED_COLOR, enable_color=C.ENABLED_COLOR):
        """

        Get Dual Methods

        Description
            Devuelve los metodos duales que se aplican en el administrador de este modelo

        :return:
        """

        if not hasattr(self, 'obj'):
            self.obj = obj
        else:
            obj = self.obj

        enable_disable_methods = [
            {'name':'enable', 'activation': True, 'color':disable_color, 'function_view':self.enable_view},
            {'name':'disable', 'activation': True, 'color':enable_color, 'function_view':self.disable_view},
        ]

        if hasattr(self, 'get_specific_dual_methods'):
            return self.get_specific_dual_methods(obj) + enable_disable_methods
        else:
            return enable_disable_methods

    def get_url(self):
        """

        Get URL

        Description
            Devuelve la URL relativa de este proceso

        :return:
        """
        info = self.model._meta.app_label, self.model._meta.model_name
        return '/admin/%s/%s/' % info

    def extra_actions(self, obj, separator=C.LINKS_SEPARATOR):
        """

        Get Status Change

        Description
            Devuelve un link para cambiar el estado de Activo
            a Inactivo o visceversa.

        """

        links = []

        for method in self.get_dual_methods(obj):

            if 'activation' in method and method['activation']:

                message = self.get_message(method['name'],'confirm')
                url = self.get_url() + '%d/%s/' % (obj.id, method['name'])
                label = self.get_message(method['name'],'label')
                color = method['color']

                link = '<a href="#" onclick="var r = confirm(\'%s\'); if (r == true){ ' \
                       'window.location.replace(\'%s\');}" ' \
                       'style="color:%s">%s</a>' % (message, url, color, label)

            if 'enquire' in method and method['enquire']:

                message = self.get_message(method['name'],'enquire')
                url = self.get_url() + '%d/%s/' % (obj.id, method['name'])
                label = self.get_message(method['name'],'label')
                color = method['color']

                default_value = ''
                if 'field_value' in method and method['field_value']:
                    default_value = getattr(obj, method['field_value'])

                link = '<a href="#" onclick="var r = prompt(\'%s\', \'%s\'); if (r != null){ ' \
                       'window.location.replace(\'%s?c=\'+r);}" ' \
                       'style="color:%s">%s</a>' % (message, default_value, url, color, label)

            links.append(link)


        return mark_safe(separator.join(links))

    # Configura el link para mostrarse en la columna
    extra_actions.short_description = 'Actions'

    def change_boolean_value_view(self, request, id, field_name):
        """

        Cambia el valor booleano de un campo

        :param request:
        :param id:
        :param field_name:
        :return:
        """
        try:

            obj = self.get_object(request, id)

            field = getattr(obj, field_name)

            if field is None:
                setattr(obj, field_name, True)
            else:
                setattr(obj, field_name, not field)

            obj.username = request.user.username
            obj.save()
            msg = '{} {}'.format(__.get_message('admin.action.%s.success' % field_name), obj.id)
            return self.response_view(request, True, '%s_view' % field_name, msg)
        except Exception as e:
            return self.response_view(request, False, '%s_view' % field_name, 'Error: {}'.format(str(e)))

    def enable_view(self, request, id):
        """

        Enable View

        Description
            Vista para habilitar una entidad

        :return:
        """
        try:
            obj = self.get_object(request, id)
            obj.enable(request.user.username)
            success_msg = __.get_message('admin.enable.success')
            return self.response_view(request, True, 'enable_view', '{}: {}'.format(success_msg, obj.id))
        except Exception as e:
            return self.response_view(request, False, 'enable_view', 'Error: {}'.format(str(e)))

    def disable_view(self, request, id):
        """

        Disable View

        Description
            Vista para deshabilitar una entidad

        :return:
        """
        try:
            obj = self.get_object(request, id)
            obj.disable(request.user.username)
            success_msg = __.get_message('admin.disable.success')
            return self.response_view(request, True, 'disable_view', '{}: {}'.format(success_msg, obj.id))
        except Exception as e:
            return self.response_view(request, False, 'disable_view', 'Error: {}'.format(str(e)))

    def response_view(self, request, response, method, extra_msg=''):
        """

        Response View

        Description
            Devuelve un HTTP Response Redirect luego de escribir un mensaje en pantalla

        :param request:
        :param response:
        :param method:
        :param extra_msg:
        :return:
        """
        if response:
            self.show_success_message(request, method, extra_msg)
        else:
            self.show_error_message(request, method, extra_msg)

        return HttpResponseRedirect(self.get_url())

    def show_success_message(self, request, method, extra_msg=''):
        """

        Show Success Message

        Description
            Muestra en pantalla el resultado de una ejecucion exitosa

        :param request:
        :param method:
        :return:
        """
        messages.info(request, '%s %s' % (self.get_message(method, 'success'), extra_msg) )

    def show_error_message(self, request, method, extra_msg=''):
        """

        Show Error Message

        Description
            Muestra en pantalla el resultado de una ejecucion fallida

        :param request:
        :param method:
        :return:
        """
        messages.error(request, '%s %s' % (self.get_message(method, 'fail'), extra_msg) )

    def get_message(self, method, response):
        """

        Get Message

        Description
            Metodo sobreescribible para obtener un mensaje

        """

        if hasattr(self, 'get_specific_message'):
            return self.get_specific_message(method, response)
        else:
            return __.get_message('admin.%s.entity.%s' % (method, response))

    def has_this_permission(self, request):
        """

        Has This Permission

        Description
            Verifica si tiene un permiso especifico

        """
        return True


# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand

import traceback

from dwll import generator

class ConsoleCommand:

    def menu(self):
        print('GENERADOR DWLL')
        print('Elija una actividad a realizar:')
        print('1. Generar nueva APP')
        print('0. Salir')

    def welcome(self):
        print('_____________________________________\n')
        print('Bienvenido a Django Web Lauch Library')
        print('_____________________________________\n')

    def main(self):
        self.welcome()
        self.menu()

        try:
            while (True):
                option = str(input('Ingrese actividad [y pulse ENTER]: '))
                if option == '0':
                    break
                elif option == '1':
                    self.option_1()
                    self.menu()
                else:
                    print('Actividad Incorrecta:', option)
        except Exception as e:
            print('Error al procesar: %s' % str(e))
            traceback.print_exc()

    def option_1(self):
        try:
            app_name = str(input('Ingrese un nombre de APP:'))
            
            option = str(input("Quiere autogenerar un modelo de ejemplo? (s/N) "))
            model_name = None
            if option == 's':
                model_name = str(input('Ingrese el nombre del modelo (tabla):'))
            
            generator.run_generator_engine('app', app_name, model_name)
        except Exception as e:
            print('Error al procesar opcion 1:', str(e))
            traceback.print_exc()

class Command(BaseCommand):
    """

    Comandos de configuracion

    """
    args = '<app app ...>'
    help = 'Console Code Generator'

    def handle(self, *args, **kwargs):
        """

        Generador de codigo por consola

        :param args:
        :param options:
        :return:
        """
        ConsoleCommand().main()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2020-06-11 22:45
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Configuration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('1', 'Activo'), ('2', 'Inactivo')], default='1', max_length=1)),
                ('creation_user', models.CharField(blank=True, max_length=64, null=True)),
                ('modification_user', models.CharField(blank=True, max_length=64, null=True)),
                ('creation_date', models.DateTimeField(blank=True, null=True)),
                ('modification_date', models.DateTimeField(blank=True, null=True)),
                ('name', models.CharField(max_length=50)),
                ('value', models.CharField(blank=True, max_length=350, null=True)),
                ('description', models.CharField(blank=True, max_length=200, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Language',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('1', 'Activo'), ('2', 'Inactivo')], default='1', max_length=1)),
                ('creation_user', models.CharField(blank=True, max_length=64, null=True)),
                ('modification_user', models.CharField(blank=True, max_length=64, null=True)),
                ('creation_date', models.DateTimeField(blank=True, null=True)),
                ('modification_date', models.DateTimeField(blank=True, null=True)),
                ('name', models.CharField(blank=True, max_length=16, null=True, unique=True)),
                ('title', models.CharField(blank=True, max_length=32, null=True, unique=True)),
                ('i18n', models.CharField(blank=True, max_length=5, null=True)),
                ('default', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('1', 'Activo'), ('2', 'Inactivo')], default='1', max_length=1)),
                ('creation_user', models.CharField(blank=True, max_length=64, null=True)),
                ('modification_user', models.CharField(blank=True, max_length=64, null=True)),
                ('creation_date', models.DateTimeField(blank=True, null=True)),
                ('modification_date', models.DateTimeField(blank=True, null=True)),
                ('name', models.CharField(blank=True, max_length=100, null=True, unique=True)),
                ('text', models.TextField(blank=True, null=True)),
                ('published', models.BooleanField(default=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MessageTraduction',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('1', 'Activo'), ('2', 'Inactivo')], default='1', max_length=1)),
                ('creation_user', models.CharField(blank=True, max_length=64, null=True)),
                ('modification_user', models.CharField(blank=True, max_length=64, null=True)),
                ('creation_date', models.DateTimeField(blank=True, null=True)),
                ('modification_date', models.DateTimeField(blank=True, null=True)),
                ('text', models.TextField(blank=True, null=True)),
                ('language', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='dwll.Language')),
                ('message', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='dwll.Message')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-
"""
.. module:: dbu - context
   :platform: Unix, Windows
   :synopsis: Contexto Principal por defecto

.. moduleauthor:: Diego Gonzalez <dgonzalez.jim@gmail.com>

"""
from . import configuration
from . import models


def load_context(request):
    """

    Load Context

    Description
        Carga las variables de contexto principales

    :param request:
    :return:
    """

    IS_TEST_MODE = configuration.isTESTMode()

    IS_MAINTENANCE = configuration.isMaintenanceMode()

    try:
        LANGUAGES = models.Language.objects.get_active()
    except:
        LANGUAGES = []

    return {
        'IS_TEST_MODE' : IS_TEST_MODE,
        'IS_MAINTENANCE' : IS_MAINTENANCE,
        'LANGUAGES' : LANGUAGES
    }


# -*- coding: utf-8 -*-
"""
.. module:: dbu-messages
   :platform: Unix, Windows
   :synopsis: Manejador de mensajes de uso comun y principal

.. moduleauthor:: Diego Gonzalez <dgonzalez.jim@gmail.com>

"""
from . import models
from .memory import memory

class LanguageManager:
    """

    Language Manager
    ===================

    Description
        Manejador de lenguajes, permite conseguir el lenguaje principal del sistema.

    """

    DEFAULT = 'spanish'

    def get_default(self):
        """
        Return the default language (spanish) 
        :return
            models.Language -- El lenguaje principal del sistema
        """
        main_languages = models.Language.objects.get_active(default=True)

        if main_languages.exists():
            return main_languages.first().name

        name = self.DEFAULT

        models.Language.objects.create(
            name=name,
            title='Espanol',
            i18n='es',
            default=True
        )

        return name

    def get_by_name(self, name):
        """
        Get the language by name
        :return
            models.Language -- El lenguaje
        """
        main_languages = models.Language.objects.get_active(name=name)

        if main_languages.count() > 0:
            return main_languages.first()

        return None

    def get_language(self, request=None):
        """
        Return the selected language or the default language
        :return
            models.Language -- El lenguaje principal del sistema
        """
        if request:
            return memory.manage(request).recover('dbu.languages.current', default_value=self.get_default())

        return self.get_default()


    def get_language_object(self, request):
        """
        Get the object of a language gived the request object, or create by default
        :return
            String -- Abreviacion i18n

        """
        lang_name = self.get_language(request)
        languages = models.Language.objects.get_active(name = lang_name)

        if languages.exists():
            return languages.first()
        else:
            return self.create_language(lang_name)

    def change_language(self, request, name):
        """
        Let to change the language gived the 'name'
        :return
            Boolean -- True if success
        """
        languages = models.Language.objects.get_active(name = name)

        if languages.count() > 0:
            first = languages.first()
            memory.manage(request).store('dbu.languages.current', first.name)
            return True, first

        return False, None

    def create_language(self, name):
        """
        Register a new language or return it if exists
        :param name:
        """
        try:
            return models.Language.objects.create(name = name, title = name)
        except Exception as e:
            languages = models.Language.objects.get_active(name = name)
            if languages.count() > 0:
                return languages.first()

            return None

    def get_languages_list(self, request):
        """
        List of active languages in list format, and with selected one checked
        """
        selected = self.get_language(request)
        return [{'is_selected':selected == lang.name, 'name':lang.name, 'title':lang.title} \
            for lang in models.Language.objects.get_active()]

languages = LanguageManager()


# -*- coding: utf-8 -*-
"""
.. module:: dwll
   :platform: Unix, Windows
   :synopsis: URLS para modulo principal

.. moduleauthor:: Diego Gonzalez <dgonzalez.jim@gmail.com>

"""
from django.urls import path, include

from . import views

urlpatterns = [

    # Standard Views
    path('language/<name>/', views.change_language, name='change-language'),
    
]



# -*- coding: utf-8 -*-
"""
.. module:: dbu-maintags
   :platform: Unix, Windows
   :synopsis: Tags del modulo DBU

.. moduleauthor:: Diego Gonzalez <dgonzalez.jim@gmail.com>

"""
from django import template
from django.utils.safestring import mark_safe

from dwll import messages
from dwll import configuration
from dwll.languages import languages

register = template.Library()

@register.simple_tag(takes_context=True)
def display_message(context, name, default=None):
    """
    Tag para mostrar un mensaje previamente almacenado en BDD
    y memoria, en pantalla formateado en HTML

    :param request: Request Actual
    :param name: Nombre del Mensaje
    :return: String -- Mensaje HTML

    """
    request = context['request']
    language = languages.get_language(request)
    return mark_safe( messages.get_message(name, language, default=default) )

@register.simple_tag(takes_context=True)
def get_configuration(context, name, default=None):
    """
    Tag para obtener una configuracion previamente almacenada en BDD
    y memoria

    :param name: Nombre de la configuracion
    :param default: Valor por defecto
    :return: String -- Valor de Configuracion

    """
    return configuration.get_value(name, default=default)

@register.simple_tag(takes_context=True)
def get_configuration_as_bool(context, name, true_value, default=None):
    """
    Tag para obtener una configuracion en formato booleano basado en el valor verdadero
    pasado en este template

    :param name: Nombre de la configuracion
    :param true_value: Valor verdadero
    :param default: Valor por defecto
    :return: String -- Mensaje HTML

    """
    return configuration.get_value(name, default=default) == true_value


@register.simple_tag(takes_context=True)
def get_languages(context):
    """
    Return the languages list
    """
    request = context['request']
    return languages.get_languages_list(request)


from django import forms

from . import messages, properties as P

import re

class CommonForm:

    """

    Utilitarios para formularios estandares bootstrap

    """

    def __init__(self):
        """

        Construye estimando que request puede ser nulo

        """
        self.request = None

    def get_module(self):
        """

        Metodo a sobrecargar

        :return:
        """
        return 'common'

    def attrs_apply(self, name, onchange, rule, msg):
        """

        Aplica otros atributos

        :param name:
        :param onchange:
        :param rule:
        :param msg:
        :return:
        """
        if onchange:
            self.fields[name].widget.attrs[onchange] = onchange

        if rule:
            self.fields[name].widget.attrs['data-rule'] = rule

        if msg:
            self.fields[name].widget.attrs['data-msg'] = msg
        else:
            self.fields[name].widget.attrs['data-msg'] = messages.get_full_message(self.request,
                                                                                   'forms.field.error.%s' % name)

    def text_config(self, name, rule=None, msg=None, onchange=None):
        """

        Text Config

        Description
            Configuracion de texto

        :param name:
        :return:
        """
        self.fields[name].widget.attrs = {
            'class':'form-control',
            'data-placement':'left',
            'placeholder': messages.get_full_message(self.request, 'forms.field.placeholder.%s' % name),
        }

        self.attrs_apply(name, onchange, rule, msg)

    def choices_config(self, name, choices, required=False, rule=None, msg=None, onchange=None):
        """

        Choices Field Config

        Description
            Configuracion de listado

        :param name:
        :return:
        """
        self.fields[name] = forms.ChoiceField(choices=choices, required=required, widget=forms.Select(attrs={
            'class': 'form-control',
            'data-placement': 'left',
            'placeholder': messages.get_full_message(self.request, 'forms.field.placeholder.%s' % name)
        }))

        self.attrs_apply(name, onchange, rule, msg)

    def datetime_config(self, name, input_formats=P.DATETIME_INPUT_FORMATS, rule=None, msg=None, required=True, onchange=None):
        """

        Date Config

        Description
            Configuracion de fechas

        :param name:
        :return:
        """
        self.fields[name] = forms.DateField(input_formats=input_formats, widget=forms.DateInput(
            attrs={
                'class': 'date form-control datepicker',
                'data-placement': 'left',
                'type':'datetime-local'
            }
        ), required=required)

        self.attrs_apply(name, onchange, rule, msg)

    def date_config(self, name, input_formats=P.DATE_INPUT_FORMATS, rule=None, msg=None, required=True, onchange=None):
        """

        Date Config

        Description
            Configuracion de fechas

        :param name:
        :return:
        """
        self.fields[name] = forms.DateField(input_formats=input_formats, widget=forms.DateInput(
            attrs={
                'class': 'date form-control datepicker',
                'data-placement': 'left',
                'type':'date'
            },
            format=('%m-%d-%Y')
        ), required=required)

        self.attrs_apply(name, onchange, rule, msg)

    def boolean_config(self, name, rule=None, msg=None, required=True, onchange=None):
        """

        Boolean Config

        Description
            Configuracion de booleanos

        :param name:
        :return:
        """
        self.fields[name] = forms.BooleanField(required=required)

        self.fields[name].widget.attrs = {
            'class': 'form-control',
            'data-placement': 'left',
            'autocomplete': 'off',
            'placeholder': messages.get_full_message(self.request, 'forms.field.placeholder.%s' % name)
        }

        self.attrs_apply(name, onchange, rule, msg)

    def decimal_config(self, name, rule=None, msg=None, step='0.01', onchange=None):
        """

        Decimal Config

        Description
            Configuracion de texto

        :param name:
        :return:
        """
        self.fields[name].widget.attrs = {
            'step':step,
            'class':'money-input form-control',
            'data-placement':'left',
            'placeholder': messages.get_full_message(self.request, 'forms.field.placeholder.%s' % name)
        }

        self.attrs_apply(name, onchange, rule, msg)

    def integer_config(self, name, rule=None, msg=None, onchange=None):
        """

        Integer Config

        Description
            Configuracion de numero entero

        :param name:
        :return:
        """
        self.fields[name].widget.attrs = {
            'step':'1',
            'class':'money-input form-control',
            'data-placement':'left',
            'placeholder': messages.get_full_message(self.request, 'forms.field.placeholder.%s' % name)
        }

        self.attrs_apply(name, onchange, rule, msg)

    def is_alphanumeric(self, text):
        """

        Valida si es solo texto y numeros sin caracteres especiales

        :param text:
        :return:
        """
        return re.match("^[a-zA-Z0-9_]*$", text) and ' ' not in text




from import_export.admin import ImportExportModelAdmin

from .modeladmin import ModelAdminMixin

class ImportExportModelAdminMixin(ImportExportModelAdmin, ModelAdminMixin):
    """

    Import Export Model
    Admin Mixin
    ===================

    Description
        Esta clase permite manejar un manager comun para todos los administradores
        de modelos del sistema que tengan posibilidad de Import Export

    """
    pass


# -*- coding: utf-8 -*-
"""
.. module:: main
   :platform: Unix, Windows
   :synopsis: Modelos de uso comun y principal

.. moduleauthor:: Diego Gonzalez <dgonzalez.jim@gmail.com>

"""

from __future__ import unicode_literals

from django.db import models

from . import mixins

class Language(mixins.AuditMixin):
    """

    Language model
    ===================

    Description
        Este modelo permite manejar los lenguajes aceptados en el sistema,
        como version inicial solo permite una codificacion unica. En caso
        de incrementar lenguajes como el Chino Mandarin u similares que
        usen caracteres especiales, se debera crear un campo de codificacion.

    """

    name = models.CharField(max_length=16, null=True, blank=True, unique=True)
    title = models.CharField(max_length=32, null=True, blank=True, unique=True)
    i18n = models.CharField(max_length=5, null=True, blank=True)
    default = models.BooleanField(default=False)

    def save(self, *args, **kwargs):
        """

        Save

        Description
            Sobrecarga el metodo save de model para formatear el nombre (name)

        """
        self.name = self.name.replace(' ','').replace('.','').replace(',','') if self.name else ''
        obj = super(Language, self).save(*args, **kwargs)
        return obj

    def __str__(self):
        return u'%s' % self.name

class Message(mixins.AuditMixin):
    """

    Messages model
    ===================

    Description
        Este modelo permite manejar los mensajes estaticos del sistema
        manteniendolos en base de datos, ademas podra ser almacenado en cache
        para limitar el acceso a base de datos. Si se agrega un cambio a este
        modelo se debera reiniciar el servidor de aplicaciones.


    """

    name = models.CharField(max_length=100, null=True, blank=True, unique=True)
    text = models.TextField(null=True, blank=True)
    published = models.BooleanField(default=True)

    def save(self, *args, **kwargs):
        """

        Save

        Description
            Sobrecarga el metodo save de model para formatear el nombre (name)

        """
        self.name = self.name.replace(' ','')
        obj = super(Message, self).save(*args, **kwargs)
        return obj

class MessageTraduction(mixins.AuditMixin):
    """

    Messages Traduction model
    ===================

    Description
        Este modelo permite almacenar la traduccion en otro idioma de un mensaje
        por defecto en caso de que el lenguaje establecido en el portal sea uno
        diferente al lenguaje por defecto (Spanish en mi caso). Si el lenguaje
        seleccionado es el lenguaje por defecto no se extra el texto de este modelo,
        se lo extrae del modelo Message del campo text.

    """
    language = models.ForeignKey('dwll.Language', on_delete=models.PROTECT)
    message = models.ForeignKey('dwll.Message', on_delete=models.PROTECT)
    text = models.TextField(null=True, blank=True)

class Configuration(mixins.AuditMixin):
    """

    Configuration model
    ===================

    Description
        Este modelo permite el almacenamiento de parametros de configuracion
        del sistema. Los parametros son accedidos por el nombre y en el helper
        correspondiente se deben mantener en cache.

    """
    name = models.CharField(max_length=50)
    value = models.CharField(max_length=350, null=True, blank=True)
    description = models.CharField(max_length=200, null=True, blank=True)

    def __str__(self):
        return u'%s' % self.name



from django.utils.decorators import method_decorator
from django.contrib.auth.decorators import login_required
from django.contrib import messages as django_messages
from django.http import HttpResponseRedirect

from dwll import messages

from django.views.generic import (
    UpdateView, 
    CreateView, 
    ListView, 
    DeleteView
)

from django.db.models import Q

from abc import (
    ABC, 
    abstractmethod
)

class LoginRequiredSecurityMixin:
    """ Clase para exigir login antes de ingresar """
    
    @method_decorator(login_required(login_url='/accounts/login/'))
    def dispatch(self, request, *args, **kwargs):
        return super().dispatch(request, *args, **kwargs)

class CRUDListView(ABC):
    """ Clase abstracta para vistas de tipo Lista """
    
    @property
    @abstractmethod
    def view_filters(self):
        pass
    
    @property
    @abstractmethod
    def view_orderedby(self):
        pass

class VirtualListView(ABC, ListView):
    """ Clase base para vistas de tipo lista """
    
    def get_queryset(self):
        queryset = self.model.objects.get_active()
        view_filters = self.view_filters
        try:
            query = None
            for field in view_filters:
                q = Q(**{"%s__icontains" % field: self.request.GET[field] })
                query = query & q if query else q
  
            queryset = queryset.filter(query)  
        except Exception as e:
            django_messages.error(self.request, str(e))

        if self.view_orderedby:
            return queryset.order_by(self.view_orderedby)
        
        return queryset

    def get_context_data(self, **kwargs):
        ctx = super().get_context_data(**kwargs)
        
        view_filters = self.view_filters

        for filter in view_filters:
            ctx[filter] = self.request.GET[filter] if filter in self.request.GET else None
        
        return ctx

class CRUDSaveView(ABC):
    """ Clase abstracta para vistas de tipo Create y Update """
    
    @abstractmethod
    def do_after_save(self, obj):
        pass
    
    @abstractmethod
    def get_success_url(self):
        pass

    @abstractmethod
    def get_fail_url(self):
        pass
    
    def get_form_kwargs(self):
        kwargs = super(CRUDSaveView, self).get_form_kwargs()
        kwargs.update({'request': self.request})
        return kwargs
    
    def form_valid(self, form, **kwargs):
        obj = form.save()
        if obj and obj.id:
            self.do_after_save(obj)
            
            obj_name = obj.__class__.__name__.lower()
            django_messages.success(self.request, 
                    messages.get_full_message(self.request, 
                    '{}.create.success'.format(obj_name)))

        return HttpResponseRedirect(self.get_success_url())

class VirtualCreateView(CRUDSaveView, CreateView):
    
    def get_success_url(self):
        return self.request.path_info
    
    def get_fail_url(self):
        return self.request.path_info
    
    def do_after_save(self, obj):
        pass

class VirtualUpdateView(CRUDSaveView, UpdateView):
    
    def get_success_url(self):
        return self.request.path_info
    
    def get_fail_url(self):
        return self.request.path_info
    
    def do_after_save(self, obj):
        pass


"""My app description"""
VERSION = (1, 0, 0)
__version__ = '.'.join([str(i) for i in VERSION])
__author__ = 'Diego Gonzalez (jimuisrael)'
__email__ = 'dgonzalez.jim@gmail.com'
__url__ = 'https://github.com/jimuisrael/dwll'
__license__ = 'BSD'
__keywords__ = ['django acelerator', 'fast login', 'messages', 'configuration', 'rehuse']


# -*- coding: utf-8 -*-
"""
.. module:: dbu-properties
   :platform: Unix, Windows
   :synopsis: Propiedades Generales

.. moduleauthor:: Diego Gonzalez <dgonzalez.jim@gmail.com>

"""

"""

Status

Description
    Estados generales a ser usados por AuditMixin para
    todos los modelos del sistema. Esto sirve para realizar
    eliminacion logica.

"""

ACTIVE = '1'
INACTIVE = '2'

# Formato de Fecha Principal
DATE_INPUT_FORMAT = '%m/%d/%Y'

"""

Date Input Format

Description
    El formato a ser usado por todos los campos de tipo
    fecha en los formularios del sistema.

"""

DATE_INPUT_FORMATS = ['%m/%d/%Y','%m/%d/%y','%Y-%m-%d']
DATETIME_INPUT_FORMATS = ['%m/%d/%Y %H:%M','%m/%d/%y %H:%M','%Y-%m-%d %H:%M']



from django.http import HttpResponseRedirect

from . import signals

from .languages import languages

def change_language(request, name):
    resp, lang = languages.change_language(request, name)
            
    if resp:
        signals.language_changed.send(sender='change_language', user=request.user, language=lang)

    return HttpResponseRedirect(request.META.get('HTTP_REFERER'))



# -*- coding: utf-8 -*-
"""
.. module:: dbu-memory
   :platform: Unix, Windows
   :synopsis: Helpers Principal de Memoria

.. moduleauthor:: Diego Gonzalez <dgonzalez.jim@gmail.com>

"""
from dwll.memory import constants as C

class Memory:
    """

    Memory
    ===================

    Description
        Helper/Manager para mantener centralizado el registro
        en memoria cache.

    """

    def __init__(self):
        """

        INIT

        :return:
        """
        self.load()
        self.request = None

    def set_request(self, request):
        self.request = request
        return self

    def load(self, step=None):
        """

        Load

        Descriptionself.assertEqual(value, "VALUE1")
            Inicializa el diccionario primario

        :return:
        """
        if not step:
            self.cache_dict = {}
            self.cache_dict[C.MAIN] = {}
        else:
            if not self.cache_dict:
                self.cache_dict = {}
            self.cache_dict[step] = {}

    def store(self, key, value, step=C.MAIN):
        """

        Store

        Description
            Almacena en la clave de un paso del diccionario
            un nuevo valor.

        :param step:
        :param key:
        :param value:
        :param request:
        :return:
        """

        if not self.cache_dict or step not in self.cache_dict:
            self.load(step)

        if self.request:
            sd = self.cache_dict[step]
            sd[key] = value
            self.request.session[C.MASTER_NAME] = self.cache_dict
            self.request.session.modified = True

        self.show()

        return value

    def recover(self, key, sub_key=None, default_value=None, step=C.MAIN):
        """

        Recover

        Description
            Recupera de la clave de un paso del diccionario
            un valor previamente almacenado.

        :param step:
        :param key:
        :param request:
        :return:
        """

        if self.request:
            self.cache_dict = self.request.session[C.MASTER_NAME] if C.MASTER_NAME in self.request.session else None

            if self.cache_dict and step in self.cache_dict and key in self.cache_dict[step]:
                if sub_key:
                    if sub_key in self.cache_dict[step][key]:
                        return self.cache_dict[step][key][sub_key]
                    return None
                else:
                    return self.cache_dict[step][key]
            elif default_value:
                return self.store(key, default_value, step=step)

        return None

    def update(self, key, value, sub_key=None, step=C.MAIN):
        """

        Update

        Description
            Actualiza o agrega un valor en un sub-diccionario almacenado
            en el diccionario principal de la memoria cache.

        :param step:
        :param key:
        :param sub_key:
        :param value:
        :param request:
        :return:
        """
        subd_dict = None

        if self.request:
            if sub_key:
                subd_dict = self.recover(key, step=step)
                subd_dict = subd_dict if subd_dict else {}
                subd_dict[sub_key] = value
            else:
                subd_dict = value

            self.store(key, subd_dict, step=step)

        self.show()

        return subd_dict

    def clean(self, request, key=None, sub_key=None, step=None):
        """

        Clean

        Description
            Limpia de la memoria o del diccionario dependiendo de
            la clave o el paso dados. Si no son dados ninguno de
            ellos se limpia toda la sesion principal.

        :param step:
        :param key:
        :param request:
        :return:
        """

        if request:
            self.cache_dict = request.session[C.MASTER_NAME] if C.MASTER_NAME in request.session else None

            if self.cache_dict:
                if step and key and sub_key:
                    try:
                        del self.cache_dict[step][key][sub_key]
                        request.session[C.MASTER_NAME] = self.cache_dict
                    except:
                        print('[Memory.clean] No se puede limpiar la cache para (', step, key, sub_key, ')')
                elif step and key:
                    try:
                        del self.cache_dict[step][key]
                        request.session[C.MASTER_NAME] = self.cache_dict
                    except:
                        print('[Memory.clean] No se puede limpiar la cache para (', step, key, ')')
                elif step:
                    try:
                        del self.cache_dict[step]
                        request.session[C.MASTER_NAME] = self.cache_dict
                    except:
                        print('[Memory.clean] No se puede limpiar la cache para (', step, ')')
                else:
                    try:
                        del request.session[C.MASTER_NAME]
                    except:
                        print('[Memory.clean] No se puede limpiar la cache total.')
            else:
                print('[Memory.clean] No hay Cache Dict')

        self.show()

    def show(self):
        """

        Show

        Description
            Imprime el diccionario de la memoria

        :return:
        """

        if self.request:
            self.cache_dict = self.request.session[C.MASTER_NAME] if C.MASTER_NAME in self.request.session else None

            if self.cache_dict:
                print('[Memory.show] - CACHE:', self.cache_dict)
            else:
                print('[Memory.show]  Vacio', self.cache_dict)

cache = Memory()

def manage(request):
    return cache.set_request(request)


from django.contrib import admin

from . import base

class ModelAdminMixin(admin.ModelAdmin, base.AdminMixinBase):
    """

    Model Admin Mixin
    ===================

    Description
        Esta clase permite manejar un manager comun para todos los administradores
        de modelos del sistema, tiene las funciones basicas de un ModelAdmin sobrecargadas.

    """

    def get_urls(self):
        """

        Get URLS

        Description
            Devuelve la lista de URLs de este Administrador

        """
       
        urls = super(ModelAdminMixin, self).get_urls()

        methods = self.get_dual_methods(self.obj if hasattr(self, 'obj') else None)

        custom_urls = []

        for m in methods:
            try:
                from django.conf.urls import url
                custom_urls.append(
                    url(r'(?P<id>\d+)/%s/$' % m['name'], self.admin_site.admin_view(m['function_view'])),
                )
            except:
                from django.urls import include, re_path
                custom_urls.append(
                    re_path(r'(?P<id>\d+)/{}/$'.format(m['name']), include(self.admin_site.admin_view(m['function_view']))),
                )
            

        return custom_urls + urls

    def get_list_display(self, request):
        """

        Get List Display

        Description
            Override

        """
        if not self.has_this_permission(request):
            self.list_display_links = (None, )

            if hasattr(self, 'protected_links') and not request.user.is_superuser:
                self.list_display = list(self.list_display)

                try:
                    for pl in self.protected_links:
                        self.list_display.remove(pl)
                except:
                    pass

                self.list_display = tuple(self.list_display)
        else:
            if hasattr(self, 'original_list_display'):
                self.list_display = self.original_list_display
                self.list_display_links = ('id', )

        return super(ModelAdminMixin, self).get_list_display(request)


# -*- coding: utf-8 -*-
"""
.. module:: dbu-admin
   :platform: Unix, Windows
   :synopsis: Administrador de Modelos de modulo dbu

.. moduleauthor:: Diego Gonzalez <dgonzalez.jim@gmail.com>

"""
from django.contrib import admin

from .models import Language, MessageTraduction, Message, Configuration

#from .admins import modeladmin

@admin.register(Language)
class LanguageAdmin(admin.ModelAdmin):#modeladmin.ModelAdminMixin
    list_display = ('id', 'name', 'default', 'modification_date', 'modification_user')

@admin.register(MessageTraduction)
class MessageTraductionAdmin(admin.ModelAdmin):#modeladmin.ModelAdminMixin
    #resource_class = impexp.MessageTraductionResource
    list_display = ('id', 'language', 'message', 'modification_date', 'modification_user')

class MessageTraductionAdminTabular(admin.TabularInline):
    model = MessageTraduction
    extra = 0
    max_num = 10
    exclude = ('status', 'creation_date', 'creation_user', 'modification_date', 'modification_user')

@admin.register(Message)
class MessageAdmin(admin.ModelAdmin):#modeladmin.ModelAdminMixin
    inlines = [MessageTraductionAdminTabular, ]
    #resource_class = impexp.MessageResource
    list_display = ('id', 'name', 'text', 'published', 'modification_date', 'modification_user')
    search_fields = ['name','text']
    list_filter = ['modification_date','creation_date']

@admin.register(Configuration)
class ConfigurationAdmin(admin.ModelAdmin):#modeladmin.ModelAdminMixin
    #resource_class = impexp.ConfigurationResource
    list_display = ('id', 'name', 'value', 'description', 'modification_date', 'modification_user')
    search_fields = ['name','value','description']



# -*- coding: utf-8 -*-
"""
.. module:: main
   :platform: Unix, Windows
   :synopsis: Mixins de uso principal y comun

.. moduleauthor:: Diego Gonzalez <dgonzalez.jim@gmail.com>

"""
from django.db import models
from django.utils import timezone

from django.db import transaction, IntegrityError

from . import properties

import traceback
import uuid

class CommonManager(models.Manager):
    """

    Common Manager
    ===================

    Description
        Esta clase permite manejar un manager comun para todos los modelos
        que extiendan de Audit Mixin y extraer consultas comunes.

    """

    def get_active(self, *args, **kwargs):
        """

        Get Active

        """
        filtered = self.get_queryset().filter(*args, **kwargs)
        filtered = filtered.filter(status=AuditMixin.ACTIVE)
        return filtered

    def get_enabled(self, *args, **kwargs):
        """

        Get First Enabled

        """
        try:
            elements = self.get_active(*args, **kwargs)
            if elements.count() > 0:
                return elements[0]
        except Exception as e:
            traceback.print_exc()

        return None

class AuditMixin(models.Model):
    """

    Audit Mixin
    ===================

    Description
        Esta clase permite manejar un modelos con campos comunes
        usados para auditoria y control de existencia.

    """

    ACTIVE = properties.ACTIVE
    INACTIVE = properties.INACTIVE

    ESTADO_CHOICES = (
        (ACTIVE, 'Activo'),
        (INACTIVE, 'Inactivo'),
    )

    status = models.CharField(max_length=1, choices=ESTADO_CHOICES, default=ACTIVE)
    creation_user = models.CharField(max_length=64, null=True, blank=True)
    modification_user = models.CharField(max_length=64, null=True, blank=True)
    creation_date = models.DateTimeField(null=True, blank=True)
    modification_date = models.DateTimeField(null=True, blank=True)

    objects = CommonManager()

    def __init__(self, *args, **kwargs):
        """

        INIT

        :param args:
        :param kwargs:
        :return:
        """
        self.username = None
        super(AuditMixin, self).__init__(*args, **kwargs)

    def save(self, *args, **kwargs):
        """

        Save

        Description
            Save Overload

        :param args:
        :param kwargs:
        :return:
        """
        request = kwargs.pop('request', None)
        user = request.user if request else None

        if not self.status:
            self.status = self.ACTIVE

        if not self.creation_user and user:
            self.creation_user = user.username[:64] if user else None

        if not self.creation_date:
            self.creation_date = timezone.now()

        self.modification_date = timezone.now()

        self.modification_user = user.username[:64] if user else self.modification_user

        obj = super(AuditMixin, self).save(*args, **kwargs)
        return obj

    def is_enabled(self):
        """

        Is Enabled

        Description
            Reg is Available

        :return:
        """
        return self.status == self.ACTIVE

    def disable(self, user=None):
        """

        Disable

        Description
            Disable a reg

        :param user:
        :return:
        """
        self.status = self.INACTIVE
        self.modification_user = user[:64] if user else ''
        self.save()

    def enable(self, user=None):
        """

        Enable

        Description
            Enable a reg

        :param user:
        :return:
        """
        self.status = self.ACTIVE
        self.modification_user = user[:64] if user else ''
        self.save()

    def get_creation_date(self):
        """

        Get Creation Time

        Description
            Get Creation Time Formated

        :return:
        """
        try:
            return self.creation_date.strftime('%d/%m/%Y %H:%M:%S')
        except:
            return str(self.creation_date)

    def get_modification_date(self):
        """

        Get Modification Time

        Description
            Get Modification Time Formated

        :return:
        """
        try:
            return self.modification_date.strftime('%d/%m/%Y %H:%M:%S')
        except:
            return str(self.modification_date)

    class Meta:
        abstract = True

class AuditMixinCode(AuditMixin):
    """

    Audit Mixin Code
    ===================

    Description
        Esta clase permite manejar un modelos con campos estandares mas un codigo alfanumerico

    """

    code = models.CharField(max_length=12, null=True, blank=True, unique=True)

    def save(self, *args, **kwargs):
        """

        Save Entity

        Description
            Guarda con codigo especial unico de entidad,
            garantizamos que sea unico

        :param args:
        :param kwargs:
        :return:
        """

        if not self.code:

            max_tries = 50
            while True:
                if max_tries > 0:
                    try:
                        self.code = self.generate_unique_id_12()
                        savepoint = transaction.savepoint()
                        super(AuditMixinCode, self).save(*args, **kwargs)
                        transaction.savepoint_commit(savepoint)
                        break
                    except IntegrityError as e:
                        print('<- [AuditMixinCode.save] Error al guardar: %s' % str(e))
                        transaction.savepoint_rollback(savepoint)
                    max_tries -= 1
                else:
                    print('<- [AuditMixinCode.save] No se pudo generar ID UNICO')
                    break
        else:
            super(AuditMixinCode, self).save(*args, **kwargs)

    @staticmethod
    def get_unique_code(max=12):
        """

        Get Unique Code

        Description
            Metodo estatico para generar codigos Unicos

        :return:
        """
        uid = uuid.uuid4().int & (1 << 64) - 1
        return str(uid)[:max]

    def generate_unique_id_12(self):
        """

        Generates Unique ID 12

        Description
            Generates a Unique ID of 12 digits

        :return:
        """
        return AuditMixinCode.get_unique_code()

    class Meta:
        abstract = True



from allauth.account.forms import LoginForm

class DwllLoginForm(LoginForm):

    def login(self, *args, **kwargs):

        # Add your own processing here.

        # You must return the original result.
        return super(DwllLoginForm, self).login(*args, **kwargs)


from setuptools import setup

import dwll

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='dwll',
    version=dwll.__version__,
    description=dwll.__doc__,
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=dwll.__keywords__,
    author=dwll.__author__,
    author_email=dwll.__email__,
    url=dwll.__url__,
    license=dwll.__license__,
    python_requires='>=3.5',
)


from bs4 import BeautifulSoup

def replace_html(file_path, var_name, new_value):
    try:
        new_text = ''
        # Open html file for reading
        with open(file_path) as html_file:
            soup = BeautifulSoup(html_file.read(), features='html.parser')
            new_text = []
            for content in soup.contents:
                new_text.append(str(content).replace(var_name, new_value))
            new_text = '\n'.join(new_text)
        # Write new contents to html file
        with open(file_path, mode='w') as new_html_file:
            new_html_file.write(new_text)

        return True, new_text
    except Exception as e:
        msg = str(e)
        return False, msg


# -*- coding: utf-8 -*-
"""
.. module:: dbu-configuration
   :platform: Unix, Windows
   :synopsis: Manager para variables de configuracion

.. moduleauthor:: Diego Gonzalez <dgonzalez.jim@gmail.com>

"""
from . import models

class ConfigurationManager:
    """

    Configuration Manager
    ===================

    Description
        Manejador de variables de configuracion, esta clase permite
        acceder a dichas variables y mantenerlas en cache.

        Solo se puede reflejar el cambio del valor de la configuracion
        desde el administrador luego de reiniciar el servidor de aplicaciones.

    """

    # Valor por defecto al crear una variable de configuracion
    DEFAULT_VALUE = '0'

    # Longitud minima de los nombres
    MAX_NAME_SIZE = 50

    # Longitud maxima de los nombres
    MAX_DESC_SIZE = 150

    # Valor por defecto usado en la configuracion del MODO
    MAIN_CONFIG_WORMODE_DEFAULT = 'TEST'

    # El sistema se encuentra en modo mantenimiento
    MAIN_CONFIG_WORMODE_MAINTENANCE = 'MANT'

    # Diccionario de Configuraciones principales de Multipasajes
    # Siempre el primero de los valores es el valor por defecto
    MAIN_CONFIGURATION = [
        {
            'name': 'common.workmode',
            'desc': 'Multipasajes Execution Mode',
            'values': [MAIN_CONFIG_WORMODE_DEFAULT, 'PROD']
        }
    ]

    def get_value(self, name, description='N/A', default=None):
        """

        Get Value

        Description
            Dado el nombre, devuelve el valor de un registro de
            configuracion, mantiene en memoria los valores.

        :return
            String -- El texto del mensaje

        """
        try:
            value = self.DEFAULT_VALUE

            if hasattr(get_value, 'values'):
                value_dict = get_value.values
            else:
                value_dict = {}

            if name in value_dict:
                value = value_dict[name]
            else:
                values = models.Configuration.objects.get_active(name=name)
                if values.count() > 0:
                    value = values.first().value
                    value_dict[name] = value
                    get_value.values = value_dict
                else:
                    self.create_default(name, description, default)
                    value = default

            return value
        except Exception as e:
            return ''

    def create_default(self, name, description, default):
        """

        Create Default

        Description
            Permite crear un parametro de configuracion por defecto en caso
            de que no exista, asi estara disponible para que el administrador
            ingrese el valor correcto

        @param name:
        @param description:
        @param default: Este es el valor por defecto para la creacion de una variable

        """
        try:
            if len(name) <= self.MAX_NAME_SIZE:
                is_config, main_config = self.is_main_config(name)
                if is_config:
                    models.Configuration.objects.create(name=name, value=main_config['values'][0],
                                                        description=self.get_main_config_description(main_config))
                else:
                    value = default if default else ''
                    models.Configuration.objects.create(name=name, value=value,
                                                        description=description[:self.MAX_DESC_SIZE]
                                                        if description else 'N/A')

        except Exception as e:
            print('<-- Error al crear la configuracion por defecto: %s' % str(e))

    def is_main_config(self, name):
        """

        Is Main Config

        Description
            Verifica si la configuracion es principal

        @param name:
        @return True si es principal
        """
        for mc in self.MAIN_CONFIGURATION:
            if mc['name'] == name:
                return True, mc
        return False, None

    def get_main_config_description(self, main_config):
        """

        Get Main Config Description

        Description
            Arma una descripcion con los posibles valores de la configuracion

        @param main_config:
        @return String -- Formato de descripcion estandar
        """
        return '%s [%s]' % (main_config['desc'], ','.join([v for v in main_config['values']]))

cmanager = ConfigurationManager()

def get_value(name, default=None, description=None):
    """

    Get Value

    Description
        Acceso rapido a las configuraciones

    :param name: El nombre unico del parametro
    :return: String -- El valor
    """
    return ConfigurationManager().get_value(name, description=description, default=default)

def get_float(name, default=None, description=None):
    """

    Get Float

    Description
        Acceso rapido a una configuracion como numero flotante

    :param name: El nombre unico del parametro
    :return: Float -- El valor
    """
    try:
        return float(ConfigurationManager().get_value(name, description=description, default=default))
    except:
        return 0

def get_integer(name, default=None, description=None):
    """

    Get Float

    Description
        Acceso rapido a una configuracion como numero entero

    :param name: El nombre unico del parametro
    :return: Integer -- El valor
    """
    try:
        return int(ConfigurationManager().get_value(name, description=description, default=default))
    except:
        return 0

def isTESTMode():
    """

    Is Test Mode

    Description
        Valida si la configuracion del sistema se encuentra en modo TEST

    :return: True si estamos en modo TEST
    """
    return str(ConfigurationManager().get_value('common.workmode', 
            default=ConfigurationManager.MAIN_CONFIG_WORMODE_DEFAULT)) == \
           str(ConfigurationManager.MAIN_CONFIG_WORMODE_DEFAULT)

def isMaintenanceMode():
    """

    Is Maintenance Mode

    Description
        Valida si la configuracion del sistema se encuentra en modo MANTENIMIENTO

    :return: True si estamos en modo MANTENIMIENTO
    """
    return str(ConfigurationManager().get_value('common.workmode')) == \
           str(ConfigurationManager.MAIN_CONFIG_WORMODE_MAINTENANCE)
